/**
 * Voicer
 * Voicer converts text into human-like speech in more than 295 voices across 50+ languages and variants.
 * Exclusively on https://1.envato.market/voicer
 *
 * @encoding        UTF-8
 * @version         3.2.3
 * @copyright       (C) 2018 - 2022 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

( function ( $ ) {

    "use strict";

    $( document ).ready( function () {

        let lastSelectetVoice;

        /** Make table great again! */
        if ( document.querySelectorAll( '#mdp-voicer-settings-language-tbl' ).length > 0 ) {

            let $langTable = $( '#mdp-voicer-settings-language-tbl' );

            $langTable.removeClass( 'hidden' );
            $langTable.DataTable( {

                /** Show entries. */
                lengthMenu: [[-1], ["All"]],

                /** Add filters to table footer. */
                initComplete: function () {
                    this.api().columns().every( function () {
                        let column = this;
                        let select = $( '#mdp-voicer-language-filter' );

                        /** Create filter only for first column. */
                        if ( column[0][0] != 0 ) {
                            return;
                        }

                        select.on( 'change', function () {

                            $( '#mdp-voicer-settings-language-tbl tbody' ).show();
                            $( '#mdp-voicer-settings-language-tbl_info' ).show();
                            $( '#mdp-voicer-settings-language-tbl_paginate' ).hide();
                            $( '#mdp-voicer-settings-language-tbl_length' ).hide();
                            $( '#mdp-voicer-settings-language-tbl thead' ).show();

                            let val = $.fn.dataTable.util.escapeRegex( $( this ).val() );
                            if ( '0' === val ) {
                                val = '';
                            }
                            column.search( val ? '^' + val + '$' : '', true, false ).draw();
                        } );

                    } );

                    // Hide all lines on first load.
                    $( '#mdp-voicer-settings-language-tbl tbody' ).hide();
                    $( '#mdp-voicer-settings-language-tbl_info' ).hide();
                    $( '#mdp-voicer-settings-language-tbl_paginate' ).hide();
                    $( '#mdp-voicer-settings-language-tbl_length' ).hide();
                    $( '#mdp-voicer-settings-language-tbl thead' ).hide();
                }
            } );

            /** Select language. */
            $( '#mdp-voicer-settings-language-tbl tbody' ).on( 'click', 'tr', function ( e ) {

                $( '#mdp-voicer-settings-language-tbl tr.selected' ).removeClass( 'selected' );
                $( this ).addClass( 'selected' );

                let voice_name = $( '#mdp-voicer-settings-language-tbl tr.selected .mdp-voice-name' ).attr( "title" );
                let lang_code = $( '#mdp-voicer-settings-language-tbl tr.selected .mdp-lang-code' ).text();

                $( '.mdp-now-used strong' ).html( voice_name );
                $( '#mdp-voicer-general-settings-language' ).val( voice_name );
                $( '#mdp-voicer-general-settings-language-code' ).val( lang_code );

                // Update Audio Sample.
                let audio = $( '.mdp-now-used audio' );
                $( '.mdp-now-used audio source:nth-child(1)' ).attr( 'src', 'https://cloud.google.com/text-to-speech/docs/audio/' + voice_name + '.mp3' );
                $( '.mdp-now-used audio source:nth-child(2)' ).attr( 'src', 'https://cloud.google.com/text-to-speech/docs/audio/' + voice_name + '.wav' );
                audio[0].pause();
                audio[0].load();

                // Save last selected voice
                lastSelectetVoice = $( this )[0].innerText;

            } );

            /** Select Language on load. */
            let index = $( '#mdp-voicer-language-filter' ).parent().data( 'mdc-index' );
            $langTable.DataTable().rows().every( function ( rowIdx, tableLoop, rowLoop ) {

                let row = this.data();

                if ( row[1].includes( $( '#mdp-voicer-general-settings-language' ).val() ) ) {

                    window.MerkulovMaterial[index].value = row[0];

                    // noinspection UnnecessaryReturnStatementJS
                    return;

                }

            } );

            /** Select change value */
            $( '#mdp-voicer-language-filter' ).on( 'change', function () {

                $( '#mdp-voicer-settings-language-tbl tr.selected' ).each( function (  ) {

                    // Remove active class from voice if another voice was selected
                    if ( ! ( lastSelectetVoice === $( this )[0].innerText ) ) {

                        $( this ).removeClass( 'selected' );

                    }

                } );

            } );

        }

        /** Drag & Drop JSON reader. */
        let $dropZone = $( '#mdp-api-key-drop-zone' );
        $dropZone.on( 'dragenter', function() {
            hideMessage();
            $( this ).addClass( 'mdp-hover' );
        } );

        $dropZone.on('dragleave', function() {
            $( this ).removeClass( 'mdp-hover' );
        } );

        /** Setup DnD voicer. */
        $dropZone.on( 'dragover', handleDragOver );

        /** Text Input to store key file. */
        let $key_input = $( '#mdp-voicer-general-settings-dnd-api-key' );

        /**
         * Read dragged file by JS.
         **/
        $dropZone.on( 'drop', function ( e ) {

            e.stopPropagation();
            e.preventDefault();

            // Show busy spinner.
            $( this ).removeClass( 'mdp-hover' );
            $dropZone.addClass( 'mdp-busy' );

            let file = e.originalEvent.dataTransfer.files[0]; // FileList object.

            /** Check is one valid JSON file. */
            if ( ! checkKeyFile( file ) ) {
                $dropZone.removeClass( 'mdp-busy' );
                return;
            }

            /** Read key file to input. */
            readFile( file )

        } );

        /**
         * Read key file to input.
         **/
        function readFile( file ) {

            let reader = new FileReader();

            /** Closure to capture the file information. */
            reader.onload = ( function( theFile ) {

                return function( e ) {

                    let json_content = e.target.result;

                    /** Check if a string is a valid JSON string. */
                    if ( ! isJSON( json_content ) ) {

                        showErrorMessage( 'Error: Uploaded file is empty or not a valid JSON file.' );

                        $dropZone.removeClass( 'mdp-busy' );
                        return;

                    }

                    /** Check if the key has required field. */
                    let key = JSON.parse( json_content );
                    if ( typeof( key.private_key ) === 'undefined' ){

                        showErrorMessage( 'Error: Your API key file looks like not valid. Please make sure you use the correct key.' );

                        $dropZone.removeClass( 'mdp-busy' );
                        return;

                    }

                    /** Encode and Save to Input. */
                    $key_input.val( btoa( json_content ) );
                    // document.getElementById( 'mdp-voicer-general-settings-api-key' ).setAttribute( 'value', btoa( json_content ) );

                    /** Hide error messages. */
                    hideMessage();

                    /** If we have long valid key in input. */
                    if ( $key_input.val().length > 1000 ) {

                        $( '#submit' ).click(); // Save settings.

                    } else {

                        showErrorMessage( 'Error: Your API key file looks like not valid. Please make sure you use the correct key.' );
                        $dropZone.removeClass( 'mdp-busy' );

                    }

                };

            } )( file );

            /** Read file as text. */
            reader.readAsText( file );

        }

        /**
         * Show upload form on click.
         **/
        let $file_input = $( '#mdp-dnd-file-input' );
        $dropZone.on( 'click', function () {

            $file_input.click();

        } );

        $file_input.on( 'change', function ( e ) {

            $dropZone.addClass( 'mdp-busy' );

            let file = e.target.files[0];

            /** Check is one valid JSON file. */
            if ( ! checkKeyFile( file ) ) {
                $dropZone.removeClass( 'mdp-busy' );
                return;
            }

            /** Read key file to input. */
            readFile( file );

        } );

        /** Show Error message under drop zone. */
        function showErrorMessage( msg ) {

            let $msgBox = $dropZone.next();

            $msgBox.addClass( 'mdp-error' ).html( msg );

        }

        /** Hide message message under drop zone. */
        function hideMessage() {

            let $msgBox = $dropZone.next();

            $msgBox.removeClass( 'mdp-error' ).html( '' );

        }

        /**
         * Check if a string is a valid JSON string.
         *
         * @param str - JSON string to check.
         **/
        function isJSON( str ) {

            try {

                JSON.parse( str );

            } catch ( e ) {

                return false;

            }

            return true;

        }

        function handleDragOver( e ) {

            e.stopPropagation();
            e.preventDefault();

        }

        /**
         * Check file is a single valid JSON file.
         *
         * @param file - JSON file to check.
         **/
        function checkKeyFile( file ) {

            /** Select only one file. */
            if ( null == file ) {

                showErrorMessage( 'Error: Failed to read file. Please try again.' );

                return false;

            }

            /** Process json file only. */
            if ( ! file.type.match( 'application/json' ) ) {

                showErrorMessage( 'Error: API Key must be a valid JSON file.' );

                return false;

            }

            return true;
        }

        /** Reset Key File. */
        $( '.mdp-reset-key-btn' ).on( 'click', function () {

            $key_input.val( '' );
            $( '#submit' ).trigger( 'click' );

        } );

        /**
         * Hide or close next tr after switch
         * @param $element
         * @param num
         * @param reverse
         */
        function switchSingle( $element, num, reverse ) {

            for ( let i = 0; i < num; i++ ) {

                if ( ! reverse ) {

                    $element.is( ':checked' ) ?
                        $element.closest( 'tr' ).nextAll( 'tr' ).eq( i ).show( 300 ) :
                        $element.closest( 'tr' ).nextAll( 'tr' ).eq( i ).hide( 300 );

                } else {

                    $element.is( ':checked' ) ?
                        $element.closest( 'tr' ).nextAll( 'tr' ).eq( i ).hide( 300 ) :
                        $element.closest( 'tr' ).nextAll( 'tr' ).eq( i ).show( 300 );

                }

            }

        }

        /**
         * Init single switch
         * @param $element
         * @param num
         * @param reverse
         */
        function initSingleSwitch( $element, num = 1, reverse = false ) {

            switchSingle( $element, num, reverse );

            $element.on( 'change', () => {

                switchSingle( $element, num, reverse );

            } );

        }

        initSingleSwitch( $( '#mdp_voicer_general_settings_multi' ), 2, true );
        initSingleSwitch( $( '#mdp_voicer_shortcode_settings_shortcode_highlight' ), 2 );
        initSingleSwitch( $( '#mdp_voicer_behaviour_settings_on_select' ), 1 );
        initSingleSwitch( $( '#mdp_voicer_behaviour_settings_on_hover' ), 2 );

    } );

} ( jQuery ) );
