<?php
/**
 * Voicer
 * Voicer converts text into human-like speech in more than 295 voices across 50+ languages and variants.
 * Exclusively on https://1.envato.market/voicer
 *
 * @encoding        UTF-8
 * @version         3.2.3
 * @copyright       (C) 2018 - 2022 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Voicer;

use Google\ApiCore\ApiException;
use Merkulove\Voicer\Unity\Tab;
use Merkulove\Voicer\Unity\Plugin;
use Merkulove\Voicer\Unity\Settings;
use Merkulove\Voicer\Unity\TabGeneral;
use Merkulove\Voicer\Unity\UI;

use Google\Cloud\TextToSpeech\V1\TextToSpeechClient;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Settings class used to modify default plugin settings.
 *
 * @since 1.0.0
 *
 **/
final class Config extends Tab {

	/**
	 * The one true Settings.
	 *
     * @since 1.0.0
     * @access private
	 * @var Config
	 **/
	private static $instance;

	/**
	 * Google languages
	 * @var \string[][]
	 */
	public static $languages = [ // TODO: Remove this and make works this with Language.php
		'da-DK'  => 'Danish (Dansk)',
		'nl-NL'  => 'Dutch (Nederlands)',
		'en-AU'  => 'English (Australian)',
		'en-GB'  => 'English (UK)',
		'en-US'  => 'English (US)',
		'fr-CA'  => 'French Canada (Français)',
		'fr-FR'  => 'French France (Français)',
		'de-DE'  => 'German (Deutsch)',
		'it-IT'  => 'Italian (Italiano)',
		'ja-JP'  => 'Japanese (日本語)',
		'ko-KR'  => 'Korean (한국어)',
		'nb-NO'  => 'Norwegian (Norsk)',
		'pl-PL'  => 'Polish (Polski)',
		'pt-BR'  => 'Portuguese Brazil (Português)',
		'pt-PT'  => 'Portuguese Portugal (Portugal)',
		'ru-RU'  => 'Russian (Русский)',
		'sk-SK'  => 'Slovak (Slovenčina)',
		'es-ES'  => 'Spanish (Español)',
		'sv-SE'  => 'Swedish (Svenska)',
		'tr-TR'  => 'Turkish (Türkçe)',
		'uk-UA'  => 'Ukrainian (Українська)',
		'ar-XA'  => 'Arabic (العربية)',
		'cs-CZ'  => 'Czech (Čeština)',
		'el-GR'  => 'Greek (Ελληνικά)',
		'en-IN'  => 'Indian English',
		'fi-FI'  => 'Finnish (Suomi)',
		'vi-VN'  => 'Vietnamese (Tiếng Việt)',
		'id-ID'  => 'Indonesian (Bahasa Indonesia)',
		'fil-PH' => 'Philippines (Filipino)',
		'hi-IN'  => 'Hindi (हिन्दी)',
		'hu-HU'  => 'Hungarian (Magyar)',
		'cmn-CN' => 'Chinese (官话)',
		'cmn-TW' => 'Taiwanese Mandarin (中文(台灣))',
		'bn-IN'  => 'Bengali (বাংলা)',
		'gu-IN'  => 'Gujarati (ગુજરાતી)',
		'kn-IN'  => 'Kannada (ಕನ್ನಡ)',
		'ml-IN'  => 'Malayalam (മലയാളം)',
		'ta-IN'  => 'Tamil (தமிழ்)',
		'te-IN'  => 'Telugu (తెలుగు)',
		'th-TH'  => 'Thai (ภาษาไทย)',
		'yue-HK' => 'Yue Chinese',
		'ro-RO'  => 'Romanian (Română)',
		'ca-ES'  => 'Catalan (Català)',
		'af-ZA'  => 'Afrikaans (South Africa)',
		'bg-BG'  => 'Bulgarian (Български)',
		'lv-LV'  => 'Latvian (Latvietis)',
		'sr-RS'  => 'Serbian (Cрпски)',
		'is-IS'  => 'Icelandic (Íslensk)',
		'es-US'  => 'US Spanish (Hispanoamericano)',
		'ms-MY'  => 'Malay (Malaysia)',
		'nl-BE'  => 'Dutch (Belgium)',
		'pa-IN'  => 'Punjabi (India)',
		'mr-IN'  => 'Marathi (India)',
		'eu-ES'  => 'Basque (Vasco)',
		'gl-ES'  => 'Galician (Galego)'
	];

    /**
     * Prepare plugin settings by modifying the default one.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function prepare_settings() {

        /** Reset API Key on fatal error. */
        if ( isset( $_GET['reset-api-key'] ) && '1' === $_GET['reset-api-key'] ) {
            $this->reset_api_key();
        }

        /** Get default plugin settings. */
        $tabs = Plugin::get_tabs();

        /** Remove 'Delete plugin, settings and data' option from Uninstall tab. */
        unset( $tabs['uninstall']['fields']['delete_plugin']['options']['plugin+settings+data'] );

        /** Set System Requirements. */
        $tabs['status']['reports']['server']['allow_url_fopen'] = false;
        $tabs['status']['reports']['server']['dom_installed'] = true;
        $tabs['status']['reports']['server']['xml_installed'] = true;
        $tabs['status']['reports']['server']['bcmath_installed'] = true;
        $tabs['status']['reports']['server']['mbstring_installed'] = true;
        $tabs['status']['reports']['server']['server_time'] = true;

        /** Shorthand access to plugin settings. */
        $options = Settings::get_instance()->options;

        /** Tab settings */
        $tabs = $this->general_tab( $tabs, $options );
        $tabs = $this->behaviour_tab( $tabs, $options );
	    $tabs = $this->design_tab( $tabs, $options );
	    $tabs = $this->shortcode_tab( $tabs, $options );

	    /** Set updated tabs. */
        Plugin::set_tabs( $tabs );

        /** Refresh settings. */
        Settings::get_instance()->get_options();

    }

	/**
     * General tab
	 * @param $tabs
	 * @param $options
	 *
	 * @return array
	 */
    private function general_tab( $tabs, $options = array() ) {

	    if ( isset( $options['dnd-api-key'] ) && $options['dnd-api-key'] ) {

		    $tabs[ 'general' ][ 'fields' ][ 'multi' ] = [
			    'type'              => 'switcher',
			    'label'             => esc_html__( 'Multilingual support', 'voicer' ),
			    'placeholder'       => esc_html__( 'Multilingual website', 'voicer' ),
			    'description'       => esc_html__( 'The Site locale will be used to generate speech. Standard-A voice will be used.', 'voicer' ),
			    'default'           => 'off',
		    ];

		    $tabs[ 'general' ][ 'fields' ][ 'current_language' ] = [
			    'type' => 'custom',
			    'render' => [ self::get_instance(), 'render_current_language' ],
			    'label' => esc_html__( 'Now used:', 'voicer' ),
			    'default' => '',
		    ];

		    $tabs[ 'general' ][ 'fields' ][ 'language' ] = [
			    'type' => 'custom',
			    'render' => [ self::get_instance(), 'render_language' ],
			    'label' => esc_html__( 'Select Language:', 'voicer' ),
			    'default' => 'en-US-Standard-B',
		    ];

		    $tabs[ 'general'] [ 'fields' ][ 'language-code' ] = [ // Hidden language code input
			    'type'              => 'text',
			    'placeholder'       => esc_html__( 'Language Code', 'voicer' ),
			    'default'           => 'en-US',
			    'attr'              => [
				    'id'        => 'mdp-voicer-general-settings-language-code',
			    ]
		    ];

		    $tabs[ 'general' ][ 'fields' ][ 'audio-profile' ] = [
			    'type' => 'select',
			    'label' => esc_html__( 'Audio Profile', 'voicer' ),
			    'placeholder' => esc_html__( 'Audio Profile', 'voicer' ),
			    'description' => esc_html__( 'Optimize the synthetic speech for playback on different types of hardware.', 'voicer' ),
			    'default' => 'handset-class-device',
			    'options' => [
				    'wearable-class-device' => esc_html__( 'Smart watches and other wearables', 'voicer' ),
				    'handset-class-device' => esc_html__( 'Smartphones', 'voicer' ),
				    'headphone-class-device' => esc_html__( 'Earbuds or headphones', 'voicer' ),
				    'small-bluetooth-speaker-class-device' => esc_html__( 'Small home speakers', 'voicer' ),
				    'medium-bluetooth-speaker-class-device' => esc_html__( 'Smart home speakers', 'voicer' ),
				    'large-home-entertainment-class-device' => esc_html__( 'Home entertainment systems', 'voicer' ),
				    'large-automotive-class-device' => esc_html__( 'Car speakers', 'voicer' ),
				    'telephony-class-application' => esc_html__( 'Interactive Voice Response', 'voicer' ),
			    ]
		    ];

		    $key = 'speaking-rate';
		    $default = 1;
		    $tabs[ 'general' ][ 'fields' ][ $key ] = [
			    'type' => 'slider',
			    'label' => esc_html__( 'Speaking Rate/Speed:', 'voicer' ),
			    'description' => esc_html__( 'Speaking rate:', 'voicer' ) .
			                     ' <strong>' .
			                     esc_html( ( isset( $options[ $key ] ) ) ? $options[ $key ] : $default ) .
			                     '</strong>',
			    'min' => 0.1,
			    'max' => 4,
			    'step' => 0.1,
			    'default' => $default,
			    'discrete' => false,
		    ];

		    $key = 'pitch';
		    $default = 0;
		    $tabs[ 'general' ][ 'fields' ][ $key ] = [
			    'type' => 'slider',
			    'label' => esc_html__( 'Pitch:', 'voicer' ),
			    'description' => esc_html__( 'Current pitch:', 'voicer' ) .
			                     ' <strong>' .
			                     esc_html( ( isset( $options[ $key ] ) ) ? $options[ $key ] : $default ) .
			                     '</strong>',
			    'min' => -20,
			    'max' => 20,
			    'step' => 1,
			    'default' => $default,
			    'discrete' => true,
		    ];

	    }

	    $tabs['general']['fields']['dnd-api-key'] = [
		    'type'              => 'custom',
		    'render'            => [ self::get_instance(), 'render_dnd_api_key' ],
		    'label'             => esc_html__( 'API Key File:', 'voicer' ),
		    'default'           => '',
	    ];

        return $tabs;

    }

	/**
	 * Behaviour tab
	 *
	 * @param $tabs
	 * @param array $options
	 *
	 * @return array|array[]
	 */
    private function behaviour_tab( $tabs, $options = array() ) {

	    $offset = 1;
	    $tabs = array_slice( $tabs, 0, $offset, true ) +
	            [ 'behaviour' => [
		            'enabled'       => true,
		            'class'         => TabGeneral::class,
		            'label'         => esc_html__( 'Behaviour', 'voicer' ),
		            'title'         => esc_html__( 'Behaviour Settings', 'voicer' ),
		            'show_title'    => true,
		            'icon'          => 'radar',
		            'fields'        => []
	            ] ] +
	            array_slice( $tabs, $offset, NULL, true );

	    $tabs[ 'behaviour' ][ 'fields' ][ 'on_select' ] = [
		    'type'              => 'switcher',
		    'label'             => esc_html__( 'Voicing selected', 'voicer' ) . ':',
		    'placeholder'       => esc_html__( 'Voicing selected', 'voicer' ),
		    'description'       => esc_html__( 'The voicing button will appear when a piece of text is selected', 'voicer' ),
		    'default'           => 'on',
	    ];

	    $tabs[ 'behaviour' ][ 'fields' ][ 'use' ] = [
		    'type'              => 'switcher',
		    'label'             => esc_html__( 'Enable for site', 'voicer' ) . ':',
		    'placeholder'       => esc_html__( 'Use everywhere', 'voicer' ),
		    'description'       => esc_html__( 'In the off mode, the Voicer will only work for blocks with CSS classes', 'voicer' ) . ' <br/><code>voicer-voice-voicename</code> ' .
		                           esc_html__( 'Learn more about this from the' ) . ' <a href="https://docs.merkulov.design/ssml-features-in-voicer-wordpress-plugin/#Voice-voice" target="_blank">' . esc_html__( 'Online Documentation' ) .  '</a>',
		    'default'           => 'on',
	    ];

	    $tabs[ 'behaviour' ][ 'fields' ][ 'on_hover' ] = [
		    'type'              => 'switcher',
		    'label'             => esc_html__( 'Voicing hover', 'voicer' ) . ':',
		    'placeholder'       => esc_html__( 'Voicing hover', 'voicer' ),
		    'description'       => esc_html__( 'The sound button will appear when hovering over the listed HTML items', 'voicer' ),
		    'default'           => 'off',
	    ];

	    $key = 'hover_delay';
	    $default = 0;
	    $value = $this->get_slider_value( 'behaviour', $key );
	    $tabs[ 'behaviour' ][ 'fields' ][$key] = [
		    'type'              => 'slider',
		    'label'             => esc_html__( 'Delay:', 'voicer' ),
		    'description'       => esc_html__( 'Hover delay:', 'voicer' ) .
		                           ' <strong>' .
		                           esc_html( is_string( $value ) ? $value : $default ) .
		                           '</strong>' . ' ' . esc_html__( 'milliseconds', 'voicer' ),
		    'min'               => 0,
		    'max'               => 3000,
		    'step'              => 100,
		    'default'           => $default,
		    'discrete'          => true,
	    ];

	    $tabs[ 'behaviour' ][ 'fields' ][ 'hover_selectors' ] = [
		    'type'              => 'textarea',
		    'label'             => esc_html__( 'CSS Selectors', 'voicer' ) . ':',
		    'placeholder'       => esc_html__( 'CSS Selectors for voicing', 'voicer' ),
		    'description'       => esc_html__( 'List CSS selectors whose contents should be voiced on hover. Use a comma to separate selectors', 'voicer' ),
		    'default'           => ''
	    ];

		$tabs[ 'behaviour' ][ 'fields' ][ 'divider' ] = [
			'type' => 'divider',
		];

		$tabs[ 'behaviour' ][ 'fields' ][ 'legacy' ] = [
			'type'              => 'switcher',
			'label'             => esc_html__( 'Legacy mode', 'voicer' ) . ':',
			'placeholder'       => esc_html__( 'Legacy mode', 'voicer' ),
			'description'       => esc_html__( 'Enable legacy mode for better compatibility. If this settings enabled environment will be compatible with PHP 7.2+', 'voicer' ),
			'default'           => 'off',
		];

        return $tabs;

    }

	/**
	 * Design tab
	 *
	 * @param $tabs
	 * @param array $options
	 *
	 * @return array|array[]
	 */
    private function design_tab( $tabs, $options = array() ) {

	    $offset = 2;
	    $tabs = array_slice( $tabs, 0, $offset, true ) +
	            ['design' => [
		            'enabled'       => true,
		            'class'         => TabGeneral::class, // Handler
		            'label'         => esc_html__( 'Design', 'voicer' ),
		            'title'         => esc_html__( 'Design Settings', 'voicer' ),
		            'show_title'    => true,
		            'icon'          => 'palette', // Icon for tab
		            'fields'        => []
	            ] ] +
	            array_slice( $tabs, $offset, NULL, true );

	    $key = 'radius';
	    $default = 100;
	    $value = $this->get_slider_value( 'design', $key );
	    $tabs['design']['fields'][$key] = [
		    'type'              => 'slider',
		    'label'             => esc_html__( 'Border radius:', 'voicer' ),
		    'description'       => esc_html__( 'Border radius:', 'voicer' ) .
		                           ' <strong>' .
		                           esc_html( is_string( $value ) ? $value : $default ) .
		                           '</strong>' .
		                           esc_html__( ' pixels', 'voicer' ),
		    'min'               => 1,
		    'max'               => 100,
		    'step'              => 1,
		    'default'           => $default,
		    'discrete'          => true,
	    ];

	    $tabs['design']['fields']['bgcolor'] = [
		    'type'              => 'colorpicker',
		    'label'             => esc_html__( 'Background Color', 'voicer' ),
		    'placeholder'       => esc_html__( 'Background Color', 'voicer' ),
		    'description'       => esc_html__( 'Select tooltip background-color', 'voicer' ),
		    'default'           => '#0253ee',
		    'attr'              => [
			    'readonly'      => 'readonly',
		    ]
	    ];

	    $tabs['design']['fields']['iconcolor'] = [
		    'type'              => 'colorpicker',
		    'label'             => esc_html__( 'Icon Color', 'voicer' ),
		    'placeholder'       => esc_html__( 'Icon Color', 'voicer' ),
		    'description'       => esc_html__( 'Select the Icon color', 'voicer' ),
		    'default'           => '#ffffff',
		    'attr'              => [
			    'readonly'      => 'readonly',
		    ]
	    ];

	    $tabs['design']['fields']['play_icon'] = [
		    'type'              => 'icon',
		    'label'             => esc_html__( 'Play icon', 'voicer' ),
		    'placeholder'       => '',
		    'description'       => esc_html__( 'Select Play Icon.', 'voicer' ),
		    'default'           => 'font-awesome/play.svg',
		    'meta'              => [
			    'font-awesome.json',
			    'material.json'
		    ]
	    ];

	    $tabs['design']['fields']['stop_icon'] = [
		    'type'              => 'icon',
		    'label'             => esc_html__( 'Stop icon', 'voicer' ),
		    'placeholder'       => '',
		    'description'       => esc_html__( 'Select Stop Icon.', 'voicer' ),
		    'default'           => 'font-awesome/stop.svg',
		    'meta'              => [
			    'font-awesome.json',
			    'material.json'
		    ]
	    ];

	    $key = 'icon_size';
	    $default = 16;
	    $value = $this->get_slider_value( 'design', $key );
	    $tabs['design']['fields'][$key] = [
		    'type'              => 'slider',
		    'label'             => esc_html__( 'Icon size:', 'voicer' ),
		    'description'       => esc_html__( 'Icons size:', 'voicer' ) .
		                           ' <strong>' .
		                           esc_html( is_string( $value ) ? $value : $default ) .
		                           '</strong>' .
		                           esc_html__( ' pixels', 'voicer' ),
		    'min'               => 1,
		    'max'               => 50,
		    'step'              => 1,
		    'default'           => $default,
		    'discrete'          => true,
	    ];

	    $key = 'icon_margin';
	    $default = 16;
	    $value = $this->get_slider_value( 'design', $key );
	    $tabs['design']['fields'][$key] = [
		    'type'              => 'slider',
		    'label'             => esc_html__( 'Icon margin:', 'voicer' ),
		    'description'       => esc_html__( 'Icons margin:', 'voicer' ) .
		                           ' <strong>' .
		                           esc_html( is_string( $value ) ? $value : $default ) .
		                           '</strong>' .
		                           esc_html__( ' pixels', 'voicer' ),
		    'min'               => 1,
		    'max'               => 50,
		    'step'              => 1,
		    'default'           => $default,
		    'discrete'          => true,
	    ];

        return $tabs;

    }

	/**
	 * Shortcode tab
	 *
	 * @param $tabs
	 * @param array $options
	 *
	 * @return array|array[]
	 */
    private function shortcode_tab( $tabs, $options = array() ) {

	    $offset = 3;
	    $tabs = array_slice( $tabs, 0, $offset, true ) +
	            ['shortcode' => [
		            'enabled'       => true,
		            'class'         => TabGeneral::class, // Handler
		            'label'         => esc_html__( 'Shortcodes', 'voicer' ),
		            'title'         => esc_html__( 'Shortcodes Settings', 'voicer' ),
		            'show_title'    => true,
		            'icon'          => 'integration_instructions',
		            'fields'        => []
	            ] ] +
	            array_slice( $tabs, $offset, NULL, true );

	    $tabs[ 'shortcode' ][ 'fields' ][ 'shortcode_mute' ] = [
		    'type'              => 'switcher',
		    'label'             => esc_html__( 'Mute shortcode', 'voicer' ),
		    'placeholder'       => esc_html__( 'Mute shortcode', 'voicer' ),
		    'description'       => esc_html__( 'Enables a', 'voicer' ) . ' <strong>[voicer-mute]</strong> ' . esc_html__( 'mute shortcode for muting some parts of text.', 'voicer' ) .
		                           ' <a href="https://docs.merkulov.design/ssml-features-in-voicer-wordpress-plugin/#mute" target="_blank">' . esc_html__( 'Read more', 'voicer' ) . '</a>',
		    'default'           => 'on',
	    ];

	    $tabs[ 'shortcode' ][ 'fields' ][ 'shortcode_break' ] = [
		    'type'              => 'switcher',
		    'label'             => esc_html__( 'Break shortcode', 'voicer' ),
		    'placeholder'       => esc_html__( 'Break shortcode', 'voicer' ),
		    'description'       => esc_html__( 'Enables a', 'voicer' ) . ' <strong>[voicer-break]</strong> ' . esc_html__( 'shortcode to add a pause to voiced text.', 'voicer' ) .
		                           ' <a href="https://docs.merkulov.design/ssml-features-in-voicer-wordpress-plugin/#pause" target="_blank">' . esc_html__( 'Read more', 'voicer' ) . '</a>',
		    'default'           => 'on',
	    ];

	    $tabs[ 'shortcode' ][ 'fields' ][ 'shortcode_voice' ] = [
		    'type'              => 'switcher',
		    'label'             => esc_html__( 'Voice shortcode', 'voicer' ),
		    'placeholder'       => esc_html__( 'Voice shortcode', 'voicer' ),
		    'description'       => esc_html__( 'Enables a', 'voicer' ) . '<strong>[voicer-voice]</strong> ' . esc_html__( 'shortcode to voicing fragments of text in another language.', 'voicer' ) .
		                           ' <a href="https://docs.merkulov.design/ssml-features-in-voicer-wordpress-plugin/#Voice-voice" target="_blank">' . esc_html__( 'Read more', 'voicer' ) . '</a>',
		    'default'           => 'on',
	    ];

	    $tabs[ 'shortcode' ][ 'fields' ][ 'shortcode_off' ] = [
		    'type'              => 'switcher',
		    'label'             => esc_html__( 'Voice off shortcode', 'voicer' ),
		    'placeholder'       => esc_html__( 'Voice off shortcode', 'voicer' ),
		    'description'       => esc_html__( 'Enables a', 'voicer' ) . ' <strong>[voicer-off]</strong> ' . esc_html__( 'shortcode to disable Voicer.', 'voicer' ) .
		                           ' <a href="https://docs.merkulov.design/ssml-features-in-voicer-wordpress-plugin/#Voice-off" target="_blank">' . esc_html__( 'Read more', 'voicer' ) . '</a>',
		    'default'           => 'on',
	    ];

	    $tabs[ 'shortcode' ][ 'fields' ][ 'shortcode_highlight' ] = [
		    'type'              => 'switcher',
		    'label'             => esc_html__( 'Highlight shortcode', 'voicer' ),
		    'placeholder'       => esc_html__( 'Highlight shortcode', 'voicer' ),
		    'description'       => esc_html__( 'Enables a', 'voicer' ) . ' <strong>[voicer-highlight]</strong> ' . esc_html__( 'shortcode that displays the play button without highlighting the text.', 'voicer' ) .
		                           ' <a href="https://docs.merkulov.design/ssml-features-in-voicer-wordpress-plugin/#highlight" target="_blank">' . esc_html__( 'Read more', 'voicer' ) . '</a>',
		    'default'           => 'on',
	    ];

	    $key = 'shortcode_highlight_delay';
	    $default = 0;
	    $value = $this->get_slider_value( 'shortcode', $key );
	    $tabs[ 'shortcode' ][ 'fields' ][ $key ] = [
		    'type'              => 'slider',
		    'label'             => esc_html__( 'Loading Delay:', 'voicer' ),
		    'description'       => esc_html__( 'Highlight shortcode loading delay:', 'voicer' ) .
		                           ' <strong>' .
		                           esc_html( is_string( $value ) ? $value : $default ) .
		                           '</strong>' . ' ' . esc_html__( 'seconds', 'voicer' ),
		    'min'               => 0,
		    'max'               => 10,
		    'step'              => .2,
		    'default'           => $default,
		    'discrete'          => false,
	    ];

	    $tabs[ 'shortcode' ][ 'fields' ][ 'shortcode_highlight_position' ] = [
		    'type'          => 'select',
		    'label'         => esc_html__( 'Tooltip position', 'voicer' ),
		    'placeholder'   => esc_html__( 'Tooltip position', 'voicer' ),
		    'description'   => esc_html__( 'Tooltip opening position for highlighted text', 'voicer' ),
		    'default' => 'top-center',
		    'options' => [
			    'top-left'      => esc_html__( 'Top Left', 'voicer' ),
			    'top-center'    => esc_html__( 'Top Center', 'voicer' ),
			    'top-right'     => esc_html__( 'Top Right', 'voicer' ),
			    'left-top'      => esc_html__( 'Left Top', 'voicer' ),
			    'left-center'   => esc_html__( 'Left Center', 'voicer' ),
			    'left-bottom'   => esc_html__( 'Left Bottom', 'voicer' ),
			    'right-top'     => esc_html__( 'Right Top', 'voicer' ),
			    'right-center'  => esc_html__( 'Right Center', 'voicer' ),
			    'right-bottom'  => esc_html__( 'Right Bottom', 'voicer' ),
			    'bottom-left'   => esc_html__( 'Bottom Left', 'voicer' ),
			    'bottom-center' => esc_html__( 'Bottom Center', 'voicer' ),
			    'bottom-right'  => esc_html__( 'Bottom Right', 'voicer' ),
		    ]
	    ];

        return $tabs;

    }

    /**
     * Get values for slider
     * @param $tab
     * @param $key
     *
     * @return false|mixed
     */
    private function get_slider_value( $tab, $key )
    {
        $tab_settings = get_option( 'mdp_voicer_' . $tab . '_settings', false );

        return ( is_array( $tab_settings ) && array_key_exists( $key, $tab_settings ) ) ?
            $tab_settings[ $key ] :
            false;

    }

    /**
     * Render Now used field.
     *
     * @param string $key - Field key.
     * @param string $tab_slug - Tab slug to which the field belongs.
     *
     * @since  1.0.0
     * @access public
     *
     * @return void
     *
     * @noinspection PhpUnused
     **/
    public function render_current_language( $key, $tab_slug ) {

        /** Shorthand for options. */
        $options = Settings::get_instance()->options;

        ?>
        <div class="mdp-now-used">
            <div>
                <strong><?php echo esc_attr( $options['language'] ); ?></strong>
            </div>
            <div>
                <audio controls="">
                    <source src="https://cloud.google.com/text-to-speech/docs/audio/<?php esc_attr_e( $options['language'] ); ?>.mp3" type="audio/mp3">
                    <source src="https://cloud.google.com/text-to-speech/docs/audio/<?php esc_attr_e( $options['language'] ); ?>.wav" type="audio/wav">
                    <?php esc_html_e( 'Your browser does not support the audio element.', 'voicer' ); ?>
                </audio>
            </div>
        </div>
        <?php

    }

    /** @noinspection PhpUnused */
    /**
     * Render Drag & Drop API Key field.
     *
     * @param string $key - Field key.
     * @param string $tab_slug - Tab slug to which the field belongs.
     *
     * @since 3.0.0
     * @access public
     **/
    public function render_dnd_api_key( $key, $tab_slug ) {

        $key_exist = false;
        if ( array_key_exists( 'dnd-api-key', Settings::get_instance()->options ) && strlen( Settings::get_instance()->options['dnd-api-key'] ) > 0 ) { $key_exist = true; }

        ?>
        <div class="mdp-dnd">
            <!--suppress HtmlFormInputWithoutLabel -->
            <div class="mdc-text-field mdc-input-width mdc-text-field--outlined mdc-hidden">
                <!--suppress HtmlFormInputWithoutLabel -->
                <input  type="text"
                        class="mdc-text-field__input"
                        name="mdp_voicer_general_settings[dnd-api-key]"
                        id="mdp-voicer-general-settings-dnd-api-key"
                        value="<?php esc_attr_e( Settings::get_instance()->options['dnd-api-key'] ); ?>"
                >
                <div class="mdc-notched-outline mdc-notched-outline--upgraded mdc-notched-outline--notched">
                    <div class="mdc-notched-outline__leading"></div>
                    <div class="mdc-notched-outline__notch">
                        <label for="mdp-voicer-general-settings-dnd-api-key" class="mdc-floating-label mdc-floating-label--float-above"><?php esc_html_e( 'API Key', 'voicer' ); ?></label>
                    </div>
                    <div class="mdc-notched-outline__trailing"></div>
                </div>
            </div>
            <div id="mdp-api-key-drop-zone" class="<?php if ( $key_exist ) : ?>mdp-key-uploaded<?php endif; ?>">
                <?php if ( $key_exist ) : ?>
                    <span class="material-icons">check_circle_outline</span><?php esc_html_e( 'API Key file exist', 'voicer' ); ?>
                    <span class="mdp-drop-zone-hover"><?php esc_html_e( 'Drop Key file here or click to upload', 'voicer' ); ?></span>
                <?php else : ?>
                    <span class="material-icons">cloud</span><?php esc_html_e( 'Drop Key file here or click to upload.', 'voicer' ); ?>
                <?php endif; ?>
            </div>
            <?php if ( $key_exist ) : ?>
                <div class="mdp-messages mdc-text-field-helper-line mdc-text-field-helper-text mdc-text-field-helper-text--persistent">
                    <?php esc_html_e( 'Drag and drop or click on the form to replace API key. |', 'voicer' ); ?>
                    <a href="#" class="mdp-reset-key-btn"><?php esc_html_e( 'Reset API Key', 'voicer' ); ?></a>
                </div>
            <?php else : ?>
                <div class="mdp-messages mdc-text-field-helper-line mdc-text-field-helper-text mdc-text-field-helper-text--persistent">
                    <?php esc_html_e( 'Drag and drop or click on the form to add API key.', 'voicer' ); ?>
                </div>
            <?php endif; ?>
            <input id="mdp-dnd-file-input" type="file" name="name" class="mdc-hidden" />
        </div>
        <?php

    }

	/**
	 * Get composer libraries versions
	 * @return string
	 */
	public function vendor_version(): string {

		$is_legacy = isset( Settings::get_instance()->options[ 'legacy' ] ) && Settings::get_instance()->options[ 'legacy' ] === 'on';
		if ( $is_legacy ) { return '7.2'; } // If legacy in on always run in lower version.

		return version_compare( PHP_VERSION, '8.1.0', '>=' ) ? '8.0' : '7.2';

	}

	/**
	 * Render settings current_language field.
	 *
	 * @return void
	 * @throws ApiException
	 * @since 2.0.0
	 * @access public
	 *
	 */
    public function render_language() {

        /** Setting custom exception handler. */
        set_exception_handler( [ ErrorHandler::class, 'exception_handler' ] );

	    /** Includes the autoloader for libraries installed with Composer. */
        require Plugin::get_path() . 'vendor/' . $this->vendor_version() . '/autoload.php';

        /** Create client object. */
        $client = new TextToSpeechClient();

        /** Perform list voices request. */
        $response = $client->listVoices();
        $voices   = $response->getVoices();

        if ( count( $voices ) === 0 ) {
            ?>
            <div class="mdp-key-error"><?php esc_html_e( 'Failed to get the list of languages. The request failed. It looks like a problem with your API Key File. Make sure that you are using the correct key file, and that the quotas have not been exceeded. If you set security restrictions on a key, make sure that the current domain is added to the exceptions.', 'voicer' ); ?></div><?php
            return;
        }

        /** Prepare Languages Options. */
        $options = [];
        $options[] = esc_html__( 'Select Language', 'voicer' );
        foreach ( $voices as $voice ) {
            $lang = Language::get_lang_by_code( $voice->getLanguageCodes() );
            $options[$lang] = $lang;
        }
        ksort( $options );

        /** Render Language select. */
        UI::get_instance()->render_select(
            $options,
            '',
            esc_html__( 'Language', 'voicer' ),
            '',
            [
                'name' => 'mdp_voicer_language_filter',
                'id' => 'mdp-voicer-language-filter'
            ]
        );

        /** Shorthand for plugin settings. */
        $options = Settings::get_instance()->options;

        ?>

        <div class="mdc-text-field-helper-line mdp-voicer-helper-padding">
            <div class="mdc-text-field-helper-text mdc-text-field-helper-text--persistent"><?php esc_html_e( 'The list includes both standard and', 'voicer' ); ?>
                <a href="https://cloud.google.com/text-to-speech/docs/wavenet"
                   target="_blank"><?php esc_html_e( 'WaveNet voices', 'voicer' ); ?></a>.
                <?php esc_html_e( 'WaveNet voices are higher quality voices with different', 'voicer' ); ?>
                <a href="https://cloud.google.com/text-to-speech/pricing"
                   target="_blank"><?php esc_html_e( 'pricing', 'voicer' ); ?></a>;
                <?php esc_html_e( 'in the list, they have the voice type "WaveNet".', 'voicer' ); ?>
            </div>
        </div>

        <table id="mdp-voicer-settings-language-tbl" class="display stripe hidden">
            <thead>
            <tr>
                <th><?php esc_html_e( 'Language', 'voicer' ); ?></th>
                <th><?php esc_html_e( 'Voice', 'voicer' ); ?></th>
                <th><?php esc_html_e( 'Gender', 'voicer' ); ?></th>
            </tr>
            </thead>
            <tbody>
            <?php foreach ( $voices as $voice ) : ?>
                <tr <?php if ( $voice->getName() === $options['language'] ) { echo 'class="selected"'; } ?>>
                    <td class="mdp-lang-name">
                        <?php echo esc_html( Language::get_lang_by_code( $voice->getLanguageCodes() ) ); // Language. ?>
                    </td>
                    <td>
                        <span class="mdp-lang-code" title="<?php echo esc_html( $voice->getLanguageCodes()[0] ); // Language Code ?>"><?php echo esc_html( $voice->getLanguageCodes()[0] ); // Language Code  ?></span>
                        -
                        <span><?php echo $this->get_voice_type( $voice->getName() ); //Voice Type ?></span>
                        -
                        <span class="mdp-voice-name" title="<?php echo esc_html( $voice->getName() ); // Voice name ?>"><?php echo esc_html( substr( $voice->getName(), -1 ) ); // Voice Variant ?></span>
                    </td>
                    <td>
                        <?php
                        /** SSML voice gender values from TextToSpeech\V1\SsmlVoiceGender. */
                        $ssmlVoiceGender = [ 'SSML_VOICE_GENDER_UNSPECIFIED', 'Male', 'Female', 'Neutral' ];

                        echo '<span title="' . esc_attr( $ssmlVoiceGender[ $voice->getSsmlGender() ], 'voicer' ) . '"><img src="' . Plugin::get_url() . 'images/' . strtolower( $ssmlVoiceGender[ $voice->getSsmlGender() ] ) . '.svg" alt="' . esc_attr( $ssmlVoiceGender[ $voice->getSsmlGender() ], 'voicer' ) . '">' . esc_html( $ssmlVoiceGender[ $voice->getSsmlGender() ] ) . '</span>'; ?>
                    </td>
                </tr>
            <?php endforeach;

            $client->close();

            ?>
            </tbody>

        </table>

        <input id="mdp-voicer-general-settings-language" type='hidden' name='mdp_voicer_general_settings[language]'
               value='<?php echo esc_attr( $options['language'] ); ?>'>
        <?php

        /** Restore previous exception handler. */
        restore_exception_handler();

    }

    /**
     * Return Voice Type.
     *
     * @param $lang_name - Google voice name.
     *
     * @return string
     * @since 2.0.0
     * @access public
     **/
	private static function get_voice_type( $lang_name ): string
	{

		$wavenet = strpos( $lang_name, 'Wavenet' );
		if ( $wavenet !== false ) {
			return wp_sprintf(
				'<img src="%s" alt="%s">%s',
				Plugin::get_url() . 'images/wavenet.svg',
				esc_html__( 'WaveNet voice', 'speaker' ),
				esc_html( 'WaveNet' )
			);
		}

		$neural = strpos( $lang_name, 'Neural' );
		if ( $neural !== false ) {
			return wp_sprintf(
				'<img src="%s" alt="%s">%s',
				Plugin::get_url() . 'images/neural.svg',
				esc_html__( 'Neural2 voice', 'speaker' ),
				esc_html( 'Neural2' )
			);
		}

		$news = strpos( $lang_name, 'News' );
		if ( $news !== false ) {
			return wp_sprintf(
				'<img src="%s" alt="%s">%s',
				Plugin::get_url() . 'images/news.svg',
				esc_html__( 'News voice', 'speaker' ),
				esc_html( 'News' )
			);
		}

		return wp_sprintf(
			'<img src="%s" alt="%s">%s',
			Plugin::get_url() . 'images/standard.svg',
			esc_html__( 'Standard voice', 'speaker' ),
			esc_html( 'Standard' )
		);

	}

    /**
     * Reset API Key on fatal error.
     *
     * @since  2.1.0
     * @access public
     * @return void
     **/
    private function reset_api_key() {

        /** Remove API Key. */
        $options = get_option( 'mdp_voicer_general_settings' );
        $options['dnd-api-key'] = '';

        /** Save new value. */
        update_option( 'mdp_voicer_general_settings', $options );

        /** Go to first tab. */
        wp_redirect( admin_url( '/admin.php?page=mdp_voicer_settings&tab=general' ) );
        exit;

    }

	/**
	 * Main Settings Instance.
	 * Insures that only one instance of Settings exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Config
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
