<?php
/**
 * Voicer
 * Voicer converts text into human-like speech in more than 295 voices across 50+ languages and variants.
 * Exclusively on https://1.envato.market/voicer
 *
 * @encoding        UTF-8
 * @version         3.2.3
 * @copyright       (C) 2018 - 2022 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Voicer;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

use Merkulove\Voicer as Voicer;
use Merkulove\Voicer\Unity\Settings;

final class Shortcodes {

	/**
	 * The one true Shortcodes.
	 *
	 * @var Shortcodes
	 * @since 1.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new Shortcodes instance.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Initializes shortcodes. */
		add_action( 'init', [$this, 'shortcodes_init'] );

	}

	/**
	 * Initializes shortcodes.
	 *
	 * @since 2.0.0
	 * @access public
	 * @return void
	 **/
	public function shortcodes_init() {

		/** Voicer Mute Shortcode. [voicer-mute]...[/voicer-mute] */
		add_shortcode( 'voicer-mute', [ $this, 'voicer_mute_shortcode' ] );

		/** Voicer Break Shortcode. [voicer-break time="2s"] */
		add_shortcode( 'voicer-break', [ $this, 'voicer_break_shortcode' ] );

		/** Voicer Voice Shortcode */
		add_shortcode( 'voicer-voice', [ $this, 'voicer_voice_shortcode' ] );

		/** Disable voicer on the page */
		add_shortcode( 'voicer-off', [ $this, 'voicer_off_shortcode' ] );

		/** Voicer highlight shortcode */
		add_shortcode( 'voicer-highlight', [ $this, 'voicer_highlight_shortcode' ] );

	}

	/**
	 * Add Voicer break by shortcode [voicer-break time="300ms"].
	 *
	 * @param $atts - An associative array of attributes specified in the shortcode.
	 *
	 * @return string
	 * @since 2.0.0
	 * @access public
	 **/
	public function voicer_break_shortcode( $atts ) {

		/** White list of options with default values. */
		$atts = shortcode_atts( [
			'time' => '500ms',
			'strength' => 'medium'
		], $atts );

		/** Extra protection from the fools */
		$atts['time'] = trim( strip_tags( $atts['time'] ) );
		$atts['strength'] = trim( strip_tags( $atts['strength'] ) );

		return '<span voicer-break="" time="' . esc_attr( $atts['time'] ) . '" strength="' . esc_attr( $atts['strength'] ) . '"></span>';

	}


	/**
	 * Add Voicer voice shortcode [voicer-voice voice=""]
	 * @param $atts
	 *
	 * @return string
	 * @since 3.0.3
	 * @access public
	 */
	public function voicer_voice_shortcode( $atts, $content ) {

		/** White list of options with default values. */
		$atts = shortcode_atts( [
			'voice' => '',
		], $atts );

		/** Extra protection from the fools */
		$atts['voice'] = trim( strip_tags( $atts['voice'] ) );

		return '<span class="voicer-voice-' . esc_attr( $atts[ 'voice' ] ) . '">' . do_shortcode( $content ) . '</span>';

	}

	/**
	 * Disable voicer shortcode
	 * @param $atts
	 * @param $content
	 *
	 * @return string
	 */
	public function voicer_off_shortcode() {

		return '<span class="voicer-off"></span>';

	}

	/**
	 * Add Voicer mute by shortcode [voicer-mute]...[/voicer-mute].
	 *
	 * @param $atts - An associative array of attributes specified in the shortcode.
	 * @param $content - Shortcode content when using the closing shortcode construct: [foo] shortcode text [/ foo].
	 *
	 * @return string
	 * @since 1.0.0
	 * @access public
	 **/
	public function voicer_mute_shortcode( $atts, $content ) {

		/** White list of options with default values. */
		/** @noinspection PhpUnusedLocalVariableInspection */
		$atts = shortcode_atts( [], $atts );

		return '<span voicer-mute="">' . do_shortcode( $content ) . '</span>';

	}

	/**
	 * Voicer highlight shortcode [voicer-highlight voice=""]...[/voicer-highlight].
	 *
	 * @param $atts
	 * @param $content
	 *
	 * @return string
	 */
	public function voicer_highlight_shortcode( $atts, $content ) {

		$options = Settings::get_instance()->options;
		if ( $options[ 'shortcode_highlight' ] === 'off' ) { return $content; }

		return wp_sprintf(
			'<span class="voicer-highlight%s" id="%s" data-position="%s">%s</span>',
			isset( $atts[ 'voice' ] ) && $atts[ 'voice' ] !== '' ? ' voicer-voice-' . esc_attr( $atts[ 'voice' ] ) : '',
			'voicer-highlight-' . $this->getUID(),
			$options[ 'shortcode_highlight_position' ],
			do_shortcode( $content )
		);

	}

	/**
	 * Generate random string
	 * @return string
	 */
	function getUID () {

		$characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$randomString = '';

		for ($i = 0; $i < 6; $i++) {
			$index = rand(0, strlen($characters) - 1);
			$randomString .= $characters[$index];
		}

		return $randomString;

	}

	/**
	 * Main Shortcodes Instance.
	 *
	 * Insures that only one instance of Shortcodes exists in memory at any one time.
	 *
	 * @static
	 * @return Shortcodes
	 * @since 2.0.0
	 **/
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Shortcodes ) ) {
			self::$instance = new Shortcodes;
		}

		return self::$instance;
	}

} // End Class Shortcodes.
